class ClientMessageEvent: ...

class ClientMessageData: ...

class ConfigWindow: ...

class CW: ...

class EventMask:
    KeyPress = ...  # type: int
    ButtonPress = ...  # type: int
    ButtonRelease = ...  # type: int
    EnterWindow = ...  # type: int
    Exposure = ...  # type: int
    StructureNotify = ...  # type: int
    FocusChange = ...  # type: int
    PropertyChange = ...  # type: int

class SetMode: ...

class StackMode: ...

class WindowClass: ...

class AccessError: ...

class DrawableError: ...

class WindowError: ...
